@@ -70,22 +70,58 @@ void librados::IoCtxImpl::queue_aio_write(AioCompletionImpl *c)
   aio_write_list_lock.Lock();
   assert(c->io == this);
   c->aio_write_seq = ++aio_write_seq;
+  ldout(client->cct, 20) << "queue_aio_write " << this << " completion " << c
+			 << " write_seq " << aio_write_seq << dendl;
   aio_write_list.push_back(&c->aio_write_list_item);
   aio_write_list_lock.Unlock();
 }
 
 void librados::IoCtxImpl::complete_aio_write(AioCompletionImpl *c)
 {
+  ldout(client->cct, 20) << "complete_aio_write " << c << dendl;
   aio_write_list_lock.Lock();
   assert(c->io == this);
   c->aio_write_list_item.remove_myself();
+  // queue async flush waiters
+  map<tid_t, std::list<AioCompletionImpl*> >::iterator waiters =
+    aio_write_waiters.find(c->aio_write_seq);
+  if (waiters != aio_write_waiters.end()) {
+    ldout(client->cct, 20) << "found " << waiters->second.size()
+			   << " waiters" << dendl;
+    for (std::list<AioCompletionImpl*>::iterator it = waiters->second.begin();
+	 it != waiters->second.end(); ++it) {
+      client->finisher.queue(new C_AioCompleteAndSafe(*it));
+      (*it)->put();
+    }
+    aio_write_waiters.erase(waiters);
+  } else {
+    ldout(client->cct, 20) << "found no waiters for tid "
+			   << c->aio_write_seq << dendl;
+  }
   aio_write_cond.Signal();
   aio_write_list_lock.Unlock();
   put();
 }
 
+void librados::IoCtxImpl::flush_aio_writes_async(AioCompletionImpl *c)
+{
+  ldout(client->cct, 20) << "flush_aio_writes_async " << this
+			 << " completion " << c << dendl;
+  Mutex::Locker l(aio_write_list_lock);
+  tid_t seq = aio_write_seq;
+  ldout(client->cct, 20) << "flush_aio_writes_async waiting on tid "
+			 << seq << dendl;
+  if (aio_write_list.empty()) {
+    client->finisher.queue(new C_AioCompleteAndSafe(c));
+  } else {
+    c->get();
+    aio_write_waiters[seq].push_back(c);
+  }
+}
+
 void librados::IoCtxImpl::flush_aio_writes()
 {
+  ldout(client->cct, 20) << "flush_aio_writes" << dendl;
   aio_write_list_lock.Lock();
   tid_t seq = aio_write_seq;
   while (!aio_write_list.empty() &&
@@ -271,7 +307,7 @@ int librados::IoCtxImpl::snap_list(vector<uint64_t> *snaps)
   const pg_pool_t *pi = objecter->osdmap->get_pg_pool(poolid);
   for (map<snapid_t,pool_snap_info_t>::const_iterator p = pi->snaps.begin();
        p != pi->snaps.end();
-       p++)
+       ++p)
     snaps->push_back(p->first);
   return 0;
 }
@@ -282,7 +318,7 @@ int librados::IoCtxImpl::snap_lookup(const char *name, uint64_t *snapid)
   const pg_pool_t *pi = objecter->osdmap->get_pg_pool(poolid);
   for (map<snapid_t,pool_snap_info_t>::const_iterator p = pi->snaps.begin();
        p != pi->snaps.end();
-       p++) {
+       ++p) {
     if (p->second.name == name) {
       *snapid = p->first;
       return 0;
@@ -659,7 +695,9 @@ int librados::IoCtxImpl::operate_read(const object_t& oid,
 
 int librados::IoCtxImpl::aio_operate_read(const object_t &oid,
 					  ::ObjectOperation *o,
-					  AioCompletionImpl *c, bufferlist *pbl)
+					  AioCompletionImpl *c,
+					  int flags,
+					  bufferlist *pbl)
 {
   Context *onack = new C_aio_Ack(c);
 
@@ -669,13 +707,14 @@ int librados::IoCtxImpl::aio_operate_read(const object_t &oid,
 
   Mutex::Locker l(*lock);
   objecter->read(oid, oloc,
-		 *o, snap_seq, pbl, 0,
+		 *o, snap_seq, pbl, flags,
 		 onack, &c->objver);
   return 0;
 }
 
 int librados::IoCtxImpl::aio_operate(const object_t& oid,
-				     ::ObjectOperation *o, AioCompletionImpl *c)
+				     ::ObjectOperation *o, AioCompletionImpl *c,
+				     const SnapContext& snap_context)
 {
   utime_t ut = ceph_clock_now(client->cct);
   /* can't write to a snapshot */
@@ -689,13 +728,15 @@ int librados::IoCtxImpl::aio_operate(const object_t& oid,
   queue_aio_write(c);
 
   Mutex::Locker l(*lock);
-  objecter->mutate(oid, oloc, *o, snapc, ut, 0, onack, oncommit, &c->objver);
+  objecter->mutate(oid, oloc, *o, snap_context, ut, 0, onack, oncommit,
+		   &c->objver);
 
   return 0;
 }
 
 int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
-				  bufferlist *pbl, size_t len, uint64_t off)
+				  bufferlist *pbl, size_t len, uint64_t off,
+				  uint64_t snapid)
 {
   if (len > (size_t) INT_MAX)
     return -EDOM;
@@ -709,13 +750,14 @@ int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
 
   Mutex::Locker l(*lock);
   objecter->read(oid, oloc,
-		 off, len, snap_seq, &c->bl, 0,
+		 off, len, snapid, &c->bl, 0,
 		 onack, &c->objver);
   return 0;
 }
 
 int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
-				  char *buf, size_t len, uint64_t off)
+				  char *buf, size_t len, uint64_t off,
+				  uint64_t snapid)
 {
   if (len > (size_t) INT_MAX)
     return -EDOM;
@@ -729,31 +771,45 @@ int librados::IoCtxImpl::aio_read(const object_t oid, AioCompletionImpl *c,
 
   Mutex::Locker l(*lock);
   objecter->read(oid, oloc,
-		 off, len, snap_seq, &c->bl, 0,
+		 off, len, snapid, &c->bl, 0,
 		 onack, &c->objver);
 
   return 0;
 }
 
+class C_ObjectOperation : public Context {
+public:
+  ::ObjectOperation m_ops;
+  C_ObjectOperation(Context *c) : m_ctx(c) {}
+  virtual void finish(int r) {
+    m_ctx->complete(r);
+  }
+private:
+  Context *m_ctx;
+};
+
 int librados::IoCtxImpl::aio_sparse_read(const object_t oid,
 					 AioCompletionImpl *c,
 					 std::map<uint64_t,uint64_t> *m,
 					 bufferlist *data_bl, size_t len,
-					 uint64_t off)
+					 uint64_t off, uint64_t snapid)
 {
   if (len > (size_t) INT_MAX)
     return -EDOM;
 
-  C_aio_sparse_read_Ack *onack = new C_aio_sparse_read_Ack(c, data_bl, m);
-  eversion_t ver;
+  Context *nested = new C_aio_Ack(c);
+  C_ObjectOperation *onack = new C_ObjectOperation(nested);
 
+  c->is_read = true;
   c->io = this;
   c->pbl = NULL;
 
+  onack->m_ops.sparse_read(off, len, m, data_bl, NULL);
+
   Mutex::Locker l(*lock);
-  objecter->sparse_read(oid, oloc,
-		 off, len, snap_seq, &c->bl, 0,
-		 onack);
+  objecter->read(oid, oloc,
+		 onack->m_ops, snap_seq, NULL, 0,
+		 onack, &c->objver);
   return 0;
 }
 
@@ -1167,33 +1223,15 @@ int librados::IoCtxImpl::sparse_read(const object_t& oid,
   if (len > (size_t) INT_MAX)
     return -EDOM;
 
-  bufferlist bl;
-
-  Mutex mylock("IoCtxImpl::read::mylock");
-  Cond cond;
-  bool done;
   int r;
-  Context *onack = new C_SafeCond(&mylock, &cond, &done, &r);
-
-  lock->Lock();
-  objecter->sparse_read(oid, oloc,
-			off, len, snap_seq, &bl, 0,
-			onack);
-  lock->Unlock();
-
-  mylock.Lock();
-  while (!done)
-    cond.Wait(mylock);
-  mylock.Unlock();
-  ldout(client->cct, 10) << "Objecter returned from read r=" << r << dendl;
+  ::ObjectOperation rd;
+  prepare_assert_ops(&rd);
+  rd.sparse_read(off, len, &m, &data_bl, NULL);
 
+  r = operate_read(oid, &rd, NULL);
   if (r < 0)
     return r;
 
-  bufferlist::iterator iter = bl.begin();
-  ::decode(m, iter);
-  ::decode(data_bl, iter);
-
   return m.size();
 }
 
@@ -1374,7 +1412,7 @@ int librados::IoCtxImpl::getxattrs(const object_t& oid,
     cond.Wait(mylock);
   mylock.Unlock();
 
-  for (map<string,bufferlist>::iterator p = aset.begin(); p != aset.end(); p++) {
+  for (map<string,bufferlist>::iterator p = aset.begin(); p != aset.end(); ++p) {
     ldout(client->cct, 10) << "IoCtxImpl::getxattrs: xattr=" << p->first << dendl;
     attrset[p->first.c_str()] = p->second;
   }
@@ -1608,36 +1646,6 @@ void librados::IoCtxImpl::C_aio_stat_Ack::finish(int r)
   c->put_unlock();
 }
 
-/////////////////////// C_aio_sparse_read_Ack //////////////////////////
-
-librados::IoCtxImpl::C_aio_sparse_read_Ack::C_aio_sparse_read_Ack(AioCompletionImpl *_c,
-								  bufferlist *data,
-								  std::map<uint64_t, uint64_t> *extents)
-  : c(_c), data_bl(data), m(extents)
-{
-  c->get();
-}
-
-void librados::IoCtxImpl::C_aio_sparse_read_Ack::finish(int r)
-{
-  c->lock.Lock();
-  c->rval = r;
-  c->ack = true;
-  c->cond.Signal();
-
-  bufferlist::iterator iter = c->bl.begin();
-  if (r >= 0) {
-    ::decode(*m, iter);
-    ::decode(*data_bl, iter);
-  }
-
-  if (c->callback_complete) {
-    c->io->client->finisher.queue(new C_AioComplete(c));
-  }
-
-  c->put_unlock();
-}
-
 //////////////////////////// C_aio_Safe ////////////////////////////////
 
 librados::IoCtxImpl::C_aio_Safe::C_aio_Safe(AioCompletionImpl *_c) : c(_c)
