@@ -140,7 +140,8 @@ struct librados::IoCtxImpl {
   int operate_read(const object_t& oid, ::ObjectOperation *o, bufferlist *pbl);
   int aio_operate(const object_t& oid, ::ObjectOperation *o,
 		  AioCompletionImpl *c, const SnapContext& snap_context);
-  int aio_operate_read(const object_t& oid, ::ObjectOperation *o, AioCompletionImpl *c, bufferlist *pbl);
+  int aio_operate_read(const object_t& oid, ::ObjectOperation *o,
+		       AioCompletionImpl *c, int flags, bufferlist *pbl);
 
   struct C_aio_Ack : public Context {
     librados::AioCompletionImpl *c;
@@ -156,16 +157,6 @@ struct librados::IoCtxImpl {
     void finish(int r);
   };
 
-  struct C_aio_sparse_read_Ack : public Context {
-    AioCompletionImpl *c;
-    bufferlist *data_bl;
-    std::map<uint64_t, uint64_t> *m;
-    C_aio_sparse_read_Ack(AioCompletionImpl *_c,
-			  bufferlist *data,
-			  std::map<uint64_t, uint64_t> *extents);
-    void finish(int r);
-  };
-
   struct C_aio_Safe : public Context {
     AioCompletionImpl *c;
     C_aio_Safe(AioCompletionImpl *_c);
