@@ -360,11 +360,10 @@ epoch_t PGMap::calc_min_last_epoch_clean() const
   if (pg_stat.empty())
     return 0;
   hash_map<pg_t,pg_stat_t>::const_iterator p = pg_stat.begin();
-  epoch_t min = p->second.get_effective_last_epoch_clean();
+  epoch_t min = p->second.last_epoch_clean;
   for (++p; p != pg_stat.end(); ++p) {
-    epoch_t lec = p->second.get_effective_last_epoch_clean();
-    if (lec < min)
-      min = lec;
+    if (p->second.last_epoch_clean < min)
+      min = p->second.last_epoch_clean;
   }
   return min;
 }
@@ -676,6 +675,13 @@ void PGMap::recovery_summary(ostream& out) const
   }
 }
 
+void PGMap::clear_delta()
+{
+  pg_sum_delta = pool_stat_t();
+  pg_sum_deltas.clear();
+  stamp_delta = ceph_clock_now(g_ceph_context);
+}
+
 void PGMap::print_summary(ostream& out) const
 {
   std::stringstream ss;
