@@ -54,7 +54,7 @@ class PaxosService {
    * If we are or have queued anything for proposal, this variable will be true
    * until our proposal has been finished.
    */
-  atomic_t proposing;
+  bool proposing;
 
  protected:
   /**
@@ -167,7 +167,7 @@ protected:
   public:
     C_Committed(PaxosService *p) : ps(p) { }
     void finish(int r) {
-      ps->proposing.set(0);
+      ps->proposing = false;
       if (r >= 0)
 	ps->_active();
       else if (r == -ECANCELED || r == -EAGAIN)
@@ -190,16 +190,15 @@ public:
    */
   PaxosService(Monitor *mn, Paxos *p, string name) 
     : mon(mn), paxos(p), service_name(name),
+      proposing(false),
       service_version(0), proposal_timer(0), have_pending(false),
       trim_version(0),
       last_committed_name("last_committed"),
       first_committed_name("first_committed"),
       last_accepted_name("last_accepted"),
       mkfs_name("mkfs"),
-      full_version_name("full"), full_latest_name("latest"),
-      cached_first_committed(0), cached_last_committed(0)
+      full_version_name("full"), full_latest_name("latest")
   {
-    proposing.set(0);
   }
 
   virtual ~PaxosService() {}
@@ -314,8 +313,6 @@ public:
    */
   bool dispatch(PaxosServiceMessage *m);
 
-  void refresh(bool *need_bootstrap);
-
   /**
    * @defgroup PaxosService_h_override_funcs Functions that should be
    *					     overridden.
@@ -338,15 +335,7 @@ public:
    *
    * @returns 'true' on success; 'false' otherwise.
    */
-  virtual void update_from_paxos(bool *need_bootstrap) = 0;
-
-  /**
-   * Hook called after all services have refreshed their state from paxos
-   *
-   * This is useful for doing any update work that depends on other
-   * service's having up-to-date state.
-   */
-  virtual void post_paxos_update() {}
+  virtual void update_from_paxos() = 0;
 
   /**
    * Init on startup
@@ -482,22 +471,6 @@ public:
    * @}
    */
 
-  /**
-   * @defgroup PaxosService_h_version_cache Variables holding cached values
-   *                                        for the most used versions (first
-   *                                        and last committed); we only have
-   *                                        to read them when the store is
-   *                                        updated, so in-between updates we
-   *                                        may very well use cached versions
-   *                                        and avoid the overhead.
-   * @{
-   */
-  version_t cached_first_committed;
-  version_t cached_last_committed;
-  /**
-   * @}
-   */
-
   /**
    * Callback list to be used whenever we are running a proposal through
    * Paxos. These callbacks will be awaken whenever the said proposal
@@ -513,7 +486,7 @@ public:
    * @returns true if we are proposing; false otherwise.
    */
   bool is_proposing() {
-    return ((int) proposing.read() == 1);
+    return proposing;
   }
 
   /**
@@ -524,29 +497,35 @@ public:
    * @returns true if in state ACTIVE; false otherwise.
    */
   bool is_active() {
-    return
-      !is_proposing() &&
-      (paxos->is_active() || paxos->is_updating());
+    return (!is_proposing() && !paxos->is_recovering()
+        && !paxos->is_locked());
   }
 
   /**
    * Check if we are readable.
    *
-   * This mirrors on the paxos check, except that we also verify that
+   * We consider that a given version @p ver is readable if:
    *
-   *  - the client hasn't seen the future relative to this PaxosService
-   *  - this service isn't proposing.
-   *  - we have committed our initial state (last_committed > 0)
+   *  - it exists (i.e., is lower than the last committed version);
+   *  - we have at least one committed version (i.e., last committed version
+   *    is greater than zero);
+   *  - our monitor is a member of the cluster (either a peon or the leader);
+   *  - we are not proposing a new version;
+   *  - the Paxos is not recovering;
+   *  - we either belong to a quorum and have a valid lease, or we belong to
+   *    a quorum of one.
    *
    * @param ver The version we want to check if is readable
    * @returns true if it is readable; false otherwise
    */
   bool is_readable(version_t ver = 0) {
-    if (ver > get_last_committed() ||
-	is_proposing() ||
-	!paxos->is_readable(0) ||
-	get_last_committed() == 0)
+    if ((ver > get_last_committed())
+	|| ((!mon->is_peon() && !mon->is_leader()))
+	|| (is_proposing() || paxos->is_recovering() || paxos->is_locked())
+	|| (get_last_committed() <= 0)
+	|| ((mon->get_quorum().size() != 1) && !paxos->is_lease_valid())) {
       return false;
+    }
     return true;
   }
 
@@ -556,16 +535,19 @@ public:
    * We consider to be writeable iff:
    *
    *  - we are not proposing a new version;
+   *  - our monitor is the leader;
+   *  - we have a valid lease;
+   *  - Paxos is not boostrapping.
+   *  - Paxos is not recovering.
    *  - we are ready to be written to -- i.e., we have a pending value.
-   *  - paxos is (active or updating)
    *
    * @returns true if writeable; false otherwise
    */
   bool is_writeable() {
-    return
-      !is_proposing() &&
-      is_write_ready() &&
-      (paxos->is_active() || paxos->is_updating());
+    return (is_active()
+        && mon->is_leader()
+        && paxos->is_lease_valid()
+        && is_write_ready());
   }
 
   /**
@@ -616,9 +598,8 @@ public:
      * Paxos; otherwise, we may assert on Paxos::wait_for_readable() if it
      * happens to be readable at that specific point in time.
      */
-    if (is_proposing() ||
-	ver > get_last_committed() ||
-	get_last_committed() == 0)
+    if (is_proposing() || (ver > get_last_committed())
+	|| (get_last_committed() <= 0))
       wait_for_finished_proposal(c);
     else
       paxos->wait_for_readable(c);
@@ -630,12 +611,12 @@ public:
    * @param c The callback to be awaken once we become writeable.
    */
   void wait_for_writeable(Context *c) {
-    if (is_proposing())
-      wait_for_finished_proposal(c);
-    else if (!is_write_ready())
-      wait_for_active(c);
-    else
+    if (!is_proposing()) {
       paxos->wait_for_writeable(c);
+      return;
+    }
+
+    wait_for_finished_proposal(c);
   }
 
   /**
@@ -652,26 +633,21 @@ public:
    */
   void trim(MonitorDBStore::Transaction *t, version_t from, version_t to);
   /**
-   * Trim our log
+   * Trim our log. This implies getting rid of versions on the k/v store.
+   * Services implementing us don't have to implement this function if they
+   * don't want to, but we won't implement it for them either.
    *
-   * Will call encode_trim_extra(), allowing services to add
-   * additional bits to the trim transaction.
+   * This function had to be inheritted from the Paxos, since the existing
+   * services made use of it. This function should be tuned for each service's
+   * needs. We have it in this interface to make sure its usage and purpose is
+   * well understood by the underlying services.
    *
    * @param first The version that should become the first one in the log.
    * @param force Optional. Each service may use it as it sees fit, but the
    *		  expected behavior is that, when 'true', we will remove all
    *		  the log versions even if we don't have a full map in store.
    */
-  void encode_trim(MonitorDBStore::Transaction *t);
-
-  /**
-   * encode service-specific extra bits into trim transaction
-   *
-   * @param tx transaction
-   * @param first new first_committed value
-   */
-  virtual void encode_trim_extra(MonitorDBStore::Transaction *tx, version_t first) {}
-
+  virtual void encode_trim(MonitorDBStore::Transaction *t);
   /**
    *
    */
@@ -896,19 +872,13 @@ public:
    *					the back store for reading purposes
    * @{
    */
-
-  /**
-   * @defgroup PaxosService_h_version_cache Obtain cached versions for this
-   *                                        service.
-   * @{
-   */
   /**
    * Get the first committed version
    *
    * @returns Our first committed version (that is available)
    */
   version_t get_first_committed() {
-    return cached_first_committed;
+    return mon->store->get(get_service_name(), first_committed_name);
   }
   /**
    * Get the last committed version
@@ -916,7 +886,7 @@ public:
    * @returns Our last committed version
    */
   version_t get_last_committed() {
-    return cached_last_committed;
+    return mon->store->get(get_service_name(), last_committed_name);
   }
   /**
    * Get our current version
@@ -926,11 +896,6 @@ public:
   version_t get_version() {
     return get_last_committed();
   }
-
-  /**
-   * @}
-   */
-
   /**
    * Get the contents of a given version @p ver
    *
