@@ -12,6 +12,7 @@
  * 
  */
 #include <sstream>
+#include <tr1/memory>
 #include "ObjectStore.h"
 #include "common/Formatter.h"
 
@@ -20,6 +21,63 @@ ostream& operator<<(ostream& out, const ObjectStore::Sequencer& s)
   return out << "osr(" << s.get_name() << " " << &s << ")";
 }
 
+unsigned ObjectStore::apply_transactions(Sequencer *osr,
+					 list<Transaction*> &tls,
+					 Context *ondisk)
+{
+  // use op pool
+  Cond my_cond;
+  Mutex my_lock("ObjectStore::apply_transaction::my_lock");
+  int r = 0;
+  bool done;
+  C_SafeCond *onreadable = new C_SafeCond(&my_lock, &my_cond, &done, &r);
+
+  queue_transactions(osr, tls, onreadable, ondisk);
+
+  my_lock.Lock();
+  while (!done)
+    my_cond.Wait(my_lock);
+  my_lock.Unlock();
+  return r;
+}
+
+template <class T>
+struct Wrapper : public Context {
+  Context *to_run;
+  T val;
+  Wrapper(Context *to_run, T val) : to_run(to_run), val(val) {}
+  void finish(int r) {
+    if (to_run)
+      to_run->complete(r);
+  }
+};
+struct RunOnDelete {
+  Context *to_run;
+  RunOnDelete(Context *to_run) : to_run(to_run) {}
+  ~RunOnDelete() {
+    if (to_run)
+      to_run->complete(0);
+  }
+};
+typedef std::tr1::shared_ptr<RunOnDelete> RunOnDeleteRef;
+int ObjectStore::queue_transactions(
+  Sequencer *osr,
+  list<Transaction*>& tls,
+  Context *onreadable,
+  Context *oncommit,
+  Context *onreadable_sync,
+  Context *oncomplete,
+  TrackedOpRef op = TrackedOpRef())
+{
+  RunOnDeleteRef _complete(new RunOnDelete(oncomplete));
+  Context *_onreadable = new Wrapper<RunOnDeleteRef>(
+    onreadable, _complete);
+  Context *_oncommit = new Wrapper<RunOnDeleteRef>(
+    oncommit, _complete);
+  return queue_transactions(osr, tls, _onreadable, _oncommit,
+			    onreadable_sync, op);
+}
+
 void ObjectStore::Transaction::dump(ceph::Formatter *f)
 {
   f->open_array_section("ops");
@@ -356,6 +414,20 @@ void ObjectStore::Transaction::dump(ceph::Formatter *f)
       break;
 
     case Transaction::OP_SPLIT_COLLECTION:
+      {
+	coll_t cid(i.get_cid());
+	uint32_t bits(i.get_u32());
+	uint32_t rem(i.get_u32());
+	coll_t dest(i.get_cid());
+	f->dump_string("op_name", "op_split_collection_create");
+	f->dump_stream("collection") << cid;
+	f->dump_stream("bits") << bits;
+	f->dump_stream("rem") << rem;
+	f->dump_stream("dest") << dest;
+      }
+      break;
+
+    case Transaction::OP_SPLIT_COLLECTION2:
       {
 	coll_t cid(i.get_cid());
 	uint32_t bits(i.get_u32());
@@ -367,6 +439,7 @@ void ObjectStore::Transaction::dump(ceph::Formatter *f)
 	f->dump_stream("rem") << rem;
 	f->dump_stream("dest") << dest;
       }
+      break;
 
     default:
       f->dump_string("op_name", "unknown");
